import mpCookie from './cookie'
import mpClient from './client'
import mpGrantType from './grant-type'
import mpResponse from './mpResponse'
import queryString from 'query-string'
import BaseApi from "../api/base-api";

export default {
    URL_AUTH_SERVER_AUTH: 'oauth/v2/auth',
    URL_AUTH_SERVER_TOKEN: 'oauth/v2/token',

    getBaseUrl() {
        let baseApi = new BaseApi()
        return baseApi._BASE_URL
    },

    /**
     * Get url for requesting the resource owner for authorization code
     *
     * @param {array} scope Authorization scope - Optional
     * @param {string} state Client state - Optional
     * @return {string} Url for getting the resource owner authorization
     */
    getAuthorizationCodeUrl(scope, state) {
        let client = mpClient.getClient()

        if (!client.redirectUri) {
            throw new Error('Missing redirect uri')
        }

        let params = {
            response_type: 'code',
            client_id: client.clientId,
            redirect_uri: client.redirectUri
        }

        // OPTIONAL
        if (scope && Array.isArray(scope) && scope.length > 0) {
            params['scope'] = scope.join(' ')
        }

        // OPTIONAL
        if (state && state.length > 0) {
            params['state'] = state
        }

        params = queryString.stringify(params)

        return this.getBaseUrl() + this.URL_AUTH_SERVER_AUTH + '?' + params
    },

    /**
     * Create AccessToken from AuthorizationCode
     *
     * @param {string} authCode
     */
    createTokenFromAuthCode(authCode, callback) {
        if (!authCode) {
            throw new Error('Missing authorization code')
        }

        let client = mpClient.getClient()

        let params = {
            code: authCode,
            redirect_uri: client.redirectUri,
            client_id: client.clientId,
            client_secret: client.secret,
            grant_type: mpGrantType.GRANT_TYPE_AUTHORIZATION_CODE
        }

        let baseApi = new BaseApi()
        let _this = this
        return new Promise(function (resolve, reject) {
            baseApi.callApi(baseApi._METHOD_POST, _this.URL_AUTH_SERVER_TOKEN, queryString.stringify(params))
                .then(result => {
                    mpCookie.setToken(result.data)
                    resolve()
                })
                .catch(error => {
                    reject(error.response)
                })
        })
    },

/*
    refreshToken(callback) {
        let client = mpClient.getClient()
        let params = {
            client_id: client.clientId,
            client_secret: client.secret,
            grant_type: mpGrantType.GRANT_TYPE_REFRESH_TOKEN,
            refresh_token: mpCookie.getRefreshToken()
        }

        let url = this.getBaseUrl() + this.URL_AUTH_SERVER_TOKEN

        Vue.http.post(url, queryString.stringify(params))
            .then(response => {
                console.log(response)
                mpCookie.setToken(response.body)
                callback(mpResponse.createResponse(true, response.body))
            })
            .catch(error => {
                callback(mpResponse.createResponse(false, error))
            })
    },
    */

    isAuthenticated() {
        /*
         if (mpCookie.getToken() === null) {
            if (mpCookie.getRefreshToken() !== null) {
                mpAuth.refreshToken(response => {
                    return response.success
                })
            }
        } else {
            return mpCookie.getToken() !== null
        }
        */
        return mpCookie.getToken()
    },
    logout() {
        mpCookie.deleteToken()
        window.top.location.href = this.getBaseUrl() + 'logout'
    }
}
